@@ -1,5 +1,5 @@
 """
-This module defines an object type which can efficiently represent
+This package defines an object type which can efficiently represent
 a bitarray.  Bitarrays are sequence types and behave very much like lists.
 
 Please find a description of this package at:
@@ -8,12 +8,12 @@ Please find a description of this package at:
 
 Author: Ilan Schnell
 """
-__version__ = '0.4.0'
+__version__ = '0.5.0'
 
 from ._bitarray import _bitarray, bits2bytes, _sysinfo
 
 
-def _btree_insert(tree, sym, ba):
+def _tree_insert(tree, sym, ba):
     """
     Insert symbol which is mapped to bitarray into tree
     """
@@ -21,7 +21,7 @@ def _btree_insert(tree, sym, ba):
     if len(ba) > 1:
         if tree[v] == []:
             tree[v] = [[], []]
-        _btree_insert(tree[v], sym, ba[1:])
+        _tree_insert(tree[v], sym, ba[1:])
     else:
         if tree[v] != []:
             raise ValueError("prefix code ambiguous")
@@ -31,7 +31,7 @@ def _mk_tree(codedict):
     # Generate tree from codedict
     tree = [[], []]
     for sym, ba in codedict.items():
-        _btree_insert(tree, sym, ba)
+        _tree_insert(tree, sym, ba)
     return tree
 
 def _check_codedict(codedict):
@@ -83,10 +83,10 @@ Append from a string, interpreting the string as machine values."""
         return self.frombytes(string.encode())
 
     def tostring(self):
-        """tostring(string)
+        """tostring()
 
-Return the string representing (machine values) of the bitarray.\n\
-When the length of the bitarray is not a multiple of 8, the few remaining\n\
+Return the string representing (machine values) of the bitarray.
+When the length of the bitarray is not a multiple of 8, the few remaining
 bits (1..7) are set to 0."""
         return self.tobytes().decode()
 
@@ -116,6 +116,20 @@ The optional argument limits the number of search results to the integer
 specified.  By default, all search results are returned."""
         return self._search(bitarray(x), limit)
 
+    def itersearch(self, x):
+        """itersearch(x)
+
+Given a bitarray x (or an object which can be converted to a bitarray),
+iterates over the start positions of x matching self."""
+        x = bitarray(x)
+        p = 0
+        while 1:
+            p = self._search_at(x, p)
+            if p is None:
+                return
+            yield p
+            p += 1
+
     def __contains__(self, other):
         """__contains__(x)
 
@@ -132,7 +146,7 @@ will be performed to determine return value."""
             except ValueError:
                 return False
         else:
-            return bool(self._search(bitarray(other), 1))
+            return self._search_at(bitarray(other), 0) is not None
 
 
 def test(verbosity=1):
