@@ -1,5 +1,7 @@
 /*
-   This file is part of the bitarray module.
+   This file is the C part of the bitarray package.  Almost all
+   functionality is implemented here.
+
    Author: Ilan Schnell
 */
 
@@ -60,7 +62,7 @@ static PyTypeObject Bitarraytype;
 
 #define BITMASK(endian, i)  (((char) 1) << ((endian) ? (7 - (i)%8) : (i)%8))
 
-/* ------------ Low level access to bits in bitarrayobject ------------- */
+/* ------------ low level access to bits in bitarrayobject ------------- */
 
 #define GETBIT(self, i)  \
     ((self)->ob_item[(i) / 8] & BITMASK((self)->endian, i) ? 1 : 0)
@@ -130,7 +132,7 @@ resize(bitarrayobject *self, idx_t nbits)
            enough to give linear-time amortized behavior over a long
            sequence of appends() in the presence of a poorly-performing
            system realloc().
-           The growth pattern is:  0, 4, 8, 16, 25, 34, 46, 56, 67, 79, ...
+           The growth pattern is:  0, 4, 8, 16, 25, 34, 44, 54, 65, 77, ...
            Note, the pattern starts out the same as for lists but then
            grows at a smaller rate so that larger bitarrays only overallocate
            by about 1/16th -- this is done because bitarrays are assumed
@@ -149,7 +151,7 @@ resize(bitarrayobject *self, idx_t nbits)
     return 0;
 }
 
-/* Create new bitarray object without initialization of buffer */
+/* create new bitarray object without initialization of buffer */
 static PyObject *
 newbitarrayobject(PyTypeObject *type, idx_t nbits, int endian)
 {
@@ -200,17 +202,29 @@ static void
 copy_n(bitarrayobject *self, idx_t a,
        bitarrayobject *other, idx_t b, idx_t n)
 {
-    /* XXX: This function could be highly optimized using memcpy for some
-       cases when self and other have same endianness, and other != self.
-    */
     idx_t i;
 
     assert (a >= 0 && b >= 0 && n >= 0);
-    /* clip n, such that only memory with ob_size is accessed */
-    if (a + n > BITS(Py_SIZE(self)))  n = BITS(Py_SIZE(self)) - a;
-    if (b + n > BITS(Py_SIZE(other))) n = BITS(Py_SIZE(other)) - b;
+    assert (a + n <= self->nbits);
+    assert (b + n <= other->nbits);
 
-    /* the different type of looping is only relevant when other is self */
+    /* XXX
+    if (self->endian == other->endian && a % 8 == 0 && b % 8 == 0 && n >= 8)
+    {
+        Py_ssize_t bytes;
+        idx_t bits;
+
+        bytes = n / 8;
+        bits = 8 * bytes;
+        copy_n(self, bits + a, other, bits + b, n - bits);
+        memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
+        return;
+    }
+    */
+
+    /* the different type of looping is only relevant when other and self
+       are the same object, i.e. when copying a piece of an bitarrayobject
+       onto itself */
     if (a < b) {
         for (i = 0; i < n; i++)             /* loop forward (delete) */
             setbit(self, i + a, GETBIT(other, i + b));
@@ -230,16 +244,11 @@ delete_n(bitarrayobject *self, idx_t start, idx_t n)
     if (n == 0)
         return 0;
 
-    copy_n(self, start, self, start + n, self->nbits - start);
-    if (resize(self, self->nbits - n) < 0)
-        return -1;
-
-    return 0;
+    copy_n(self, start, self, start + n, self->nbits - start - n);
+    return resize(self, self->nbits - n);
 }
 
-/* at start, insert n bits into self.
-   The vaules of the inserted bits are unspecified.
-*/
+/* starting at start, insert n (uninitialized) bits into self */
 static int
 insert_n(bitarrayobject *self, idx_t start, idx_t n)
 {
@@ -250,8 +259,7 @@ insert_n(bitarrayobject *self, idx_t start, idx_t n)
 
     if (resize(self, self->nbits + n) < 0)
         return -1;
-
-    copy_n(self, start + n, self, start, self->nbits - start);
+    copy_n(self, start + n, self, start, self->nbits - start - n);
     return 0;
 }
 
@@ -336,7 +344,7 @@ bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
     return 0;
 }
 
-/* set the bits from start to stop (excluded) in self to val */
+/* set the bits from start to stop (excluding) in self to val */
 static void
 setrange(bitarrayobject *self, idx_t start, idx_t stop, int val)
 {
@@ -357,7 +365,7 @@ bytereverse(bitarrayobject *self)
     int c;
 
     if (!setup) {
-        /* Setup a translation table, which maps each byte to it's
+        /* setup a translation table, which maps each byte to it's
            reversed: trans = {0, 128, 64, 192, 32, 160, ..., 255}
         */
         int j, k;
@@ -399,7 +407,7 @@ count(bitarrayobject *self)
     int c;
 
     if (!setup) {
-        /* Setup a translation table, which maps each byte to it's
+        /* setup a translation table, which maps each byte to it's
            bit count: trans = {0, 1, 1, 2, 1, 2, 2, 3, 1, ..., 8}
         */
         int j, k;
@@ -421,7 +429,7 @@ count(bitarrayobject *self)
     return res;
 }
 
-/* Return index of first occurrence of vi, -1 when x is not in found. */
+/* return index of first occurrence of vi, -1 when x is not in found. */
 static idx_t
 findfirst(bitarrayobject *self, int vi)
 {
@@ -456,7 +464,8 @@ set_item(bitarrayobject *self, idx_t i, PyObject *v)
     long vi;
 
     assert (i >= 0 && i < self->nbits);
-    if ((vi = PyObject_IsTrue(v)) < 0)
+    vi = PyObject_IsTrue(v);
+    if (vi < 0)
         return -1;
 
     setbit(self, i, vi);
@@ -593,7 +602,8 @@ extend_string(bitarrayobject *self, PyObject *string, enum conv_tp conv)
     int vi = 0;
 
     assert (PyString_Check(string));
-    if ((strlen = PyString_Size(string)) == 0)
+    strlen = PyString_Size(string);
+    if (strlen == 0)
         return 0;
 
     if (resize(self, self->nbits + strlen) < 0)
@@ -631,7 +641,8 @@ extend_rawstring(bitarrayobject *self, PyObject *string)
     char *str;
 
     assert (PyString_Check(string) && self->nbits % 8 == 0);
-    if ((strlen = PyString_Size(string)) == 0)
+    strlen = PyString_Size(string);
+    if (strlen == 0)
         return 0;
 
     if (resize(self, self->nbits + BITS(strlen)) < 0)
@@ -727,7 +738,7 @@ getIndex(PyObject *v, idx_t *i)
     return 1;
 }
 
-/* This is PySlice_GetIndicesEx() with Py_ssize_t replaced by idx_t */
+/* this is PySlice_GetIndicesEx() with Py_ssize_t replaced by idx_t */
 static int
 slice_GetIndicesEx(PySliceObject *r, idx_t length,
                    idx_t *start, idx_t *stop, idx_t *step, idx_t *slicelength)
@@ -782,9 +793,9 @@ slice_GetIndicesEx(PySliceObject *r, idx_t length,
     return 0;
 }
 
-/****************************************************************************
+/**************************************************************************
                          Implementation of API methods
-****************************************************************************/
+ **************************************************************************/
 
 static PyObject *
 bitarray_length(bitarrayobject *self)
@@ -853,10 +864,12 @@ bitarray_index(bitarrayobject *self, PyObject *v)
     idx_t i;
     long vi;
 
-    if ((vi = PyObject_IsTrue(v)) < 0)
+    vi = PyObject_IsTrue(v);
+    if (vi < 0)
         return NULL;
 
-    if ((i = findfirst(self, vi)) < 0) {
+    i = findfirst(self, vi);
+    if (i < 0) {
         PyErr_SetString(PyExc_ValueError, "index(x): x not in bitarray");
         return NULL;
     }
@@ -902,7 +915,7 @@ bitarray_search(bitarrayobject *self, PyObject *args)
     xa = (bitarrayobject *) x;
 
     if (xa->nbits == 0) {
-        PyErr_SetString(PyExc_TypeError, "can't search for empty bitarray");
+        PyErr_SetString(PyExc_ValueError, "can't search for empty bitarray");
         goto error;
     }
     list = PyList_New(0);
@@ -914,7 +927,7 @@ bitarray_search(bitarrayobject *self, PyObject *args)
             if (GETBIT(self, p + n) != GETBIT(xa, n))
                 goto next;
 
-        /* We have a match, append the position to the list */
+        /* we have a match, append the position to the list */
         item = PyLong_FromLongLong(p);
         if (item == NULL || PyList_Append(list, item) < 0)
             goto error;
@@ -923,7 +936,7 @@ bitarray_search(bitarrayobject *self, PyObject *args)
         if (limit > 0 && PyList_Size(list) >= limit)
             break;
     next:
-        ; /* Do nothing */
+        ; /* do nothing */
     }
     return list;
 error:
@@ -936,6 +949,49 @@ PyDoc_STRVAR(search_doc,
 "_search(x, limit)  like search but x has to be a bitarray.");
 
 
+static PyObject *
+bitarray_search_at(bitarrayobject *self, PyObject *args)
+{
+    PyObject *x;
+    bitarrayobject *xa;
+    idx_t p, n;
+
+    if (!PyArg_ParseTuple(args, "OL:_search_at", &x, &p))
+        return NULL;
+
+    assert(bitarray_Check(x));
+    xa = (bitarrayobject *) x;
+
+    if (xa->nbits == 0) {
+        PyErr_SetString(PyExc_ValueError, "can't search for empty bitarray");
+        return NULL;
+    }
+    if (xa->nbits > self->nbits)
+        Py_RETURN_NONE;
+
+    if (p < 0) {
+        PyErr_SetString(PyExc_ValueError, "positive start value expected");
+        return NULL;
+    }
+
+    for (; p < self->nbits - xa->nbits + 1; p++) {
+        for (n = 0; n < xa->nbits; n++)
+            if (GETBIT(self, p + n) != GETBIT(xa, n))
+                goto next;
+
+        /* we have a match, return the position */
+        return PyLong_FromLongLong(p);
+    next:
+        ; /* do nothing */
+    }
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(search_at_doc,
+"_search_at(x, start)  search for bitarray x starting at start, and return\n\
+the index where the bitarray x is found (or None if x is not found).");
+
+
 static PyObject *
 bitarray_buffer_info(bitarrayobject *self)
 {
@@ -1072,11 +1128,11 @@ bitarray_reverse(bitarrayobject *self)
 
     m = self->nbits - 1;
 
-    /* Reverse the upper half onto the lower half. */
+    /* reverse the upper half onto the lower half. */
     for (i = 0; i < tt->nbits; i++)
         setbit(self, i, GETBIT(self, m - i));
 
-    /* Revert the stored away lower half onto the upper half. */
+    /* revert the stored away lower half onto the upper half. */
     for (i = 0; i < tt->nbits; i++)
         setbit(self, m - i, GETBIT(tt, i));
 #undef tt
@@ -1231,7 +1287,7 @@ bitarray_fromfile(bitarrayobject *self, PyObject *args)
             return NULL;
         }
 
-        nread=PyBytes_Size(result);
+        nread = PyBytes_Size(result);
 
         t = self->nbits;
         p = setunused(self);
@@ -1253,7 +1309,8 @@ bitarray_fromfile(bitarrayobject *self, PyObject *args)
             PyErr_SetString(PyExc_EOFError, "not enough items read");
             return NULL;
         }
-        delete_n(self, t, p);
+        if (delete_n(self, t, p) < 0)
+            return NULL;
         Py_DECREF(result);
     }
 
@@ -1301,7 +1358,7 @@ bitarray_fromfile(bitarrayobject *self, PyObject *args)
     if (nbytes == 0)
         Py_RETURN_NONE;
 
-    /* File exists and there are more than zero bytes to read */
+    /* file exists and there are more than zero bytes to read */
     t = self->nbits;
     p = setunused(self);
     self->nbits += p;
@@ -1319,7 +1376,8 @@ bitarray_fromfile(bitarrayobject *self, PyObject *args)
         return NULL;
     }
 
-    delete_n(self, t, p);
+    if (delete_n(self, t, p) < 0)
+        return NULL;
     Py_RETURN_NONE;
 }
 #endif
@@ -1337,6 +1395,7 @@ static PyObject *
 bitarray_tofile(bitarrayobject *self, PyObject *f)
 {
     PyObject *writer, *value, *args, *result;
+
     if (f == NULL) {
         PyErr_SetString(PyExc_TypeError, "writeobject with NULL file");
         return NULL;
@@ -1408,7 +1467,8 @@ bitarray_tolist(bitarrayobject *self)
     PyObject *list;
     idx_t i;
 
-    if ((list = PyList_New((Py_ssize_t) self->nbits)) == NULL)
+    list = PyList_New((Py_ssize_t) self->nbits);
+    if (list == NULL)
         return NULL;
 
     for (i = 0; i < self->nbits; i++)
@@ -1441,7 +1501,8 @@ bitarray_frombytes(bitarrayobject *self, PyObject *string)
     if (extend_rawstring(self, string) < 0)
         return NULL;
 
-    delete_n(self, t, p);
+    if (delete_n(self, t, p) < 0)
+        return NULL;
     Py_RETURN_NONE;
 }
 
@@ -1469,16 +1530,16 @@ bits (1..7) are set to 0.");
 static PyObject *
 bitarray_to01(bitarrayobject *self)
 {
+#ifdef IS_PY3K
     PyObject *string, *unpacked;
 
     unpacked = unpack(self, '0', '1');
-#ifdef IS_PY3K
     string = PyUnicode_FromEncodedObject(unpacked, NULL, NULL);
     Py_DECREF(unpacked);
-#else
-    string = unpacked;
-#endif
     return string;
+#else
+    return unpack(self, '0', '1');
+#endif
 }
 
 PyDoc_STRVAR(to01_doc,
@@ -1614,7 +1675,6 @@ bitarray_pop(bitarrayobject *self, PyObject *args)
     vi = GETBIT(self, i);
     if (delete_n(self, i, 1) < 0)
         return NULL;
-
     return PyBool_FromLong(vi);
 }
 
@@ -1731,7 +1791,8 @@ setslice(bitarrayobject *self, PySliceObject *slice, PyObject *v)
         return 0;
     }
     else if (PyBool_Check(v)) {
-        if ((vi = PyObject_IsTrue(v)) < 0)
+        vi = PyObject_IsTrue(v);
+        if (vi < 0)
             return -1;
 
         for (i = 0, j = start; i < slicelength; i++, j += step)
@@ -1815,7 +1876,7 @@ bitarray_delitem(bitarrayobject *self, PyObject *a)
                 return NULL;
             Py_RETURN_NONE;
         }
-        /* This is the only complicated part when step > 1 */
+        /* this is the only complicated part when step > 1 */
         for (i = j = start; i < self->nbits; i++)
             if ((i - start) % step != 0 || i >= stop) {
                 setbit(self, j, GETBIT(self, i));
@@ -1951,7 +2012,7 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
         PyErr_SetString(PyExc_TypeError, "iterable object expected");
         return NULL;
     }
-    /* Extend self with the bitarrays from codedict */
+    /* extend self with the bitarrays from codedict */
     while ((symbol = PyIter_Next(iter)) != NULL) {
         bits = PyDict_GetItem(codedict, symbol);
         if (bits == NULL) {
@@ -1977,16 +2038,17 @@ PyDoc_STRVAR(encode_doc,
 "_encode(code, iterable) is like the encode method without code checking");
 
 
-/* Return the leave node resulting from traversing the binary tree,
+/* return the leave node resulting from traversing the (binary) tree,
    or, when the iteration is finished, NULL
 */
 static PyObject *
-btree_traverse(PyObject *iter, PyObject *tree)
+tree_traverse(PyObject *iter, PyObject *tree)
 {
     PyObject *subtree, *v;
     long vi;
 
-    if ((v = PyIter_Next(iter)) == NULL)  /* stop iterator */
+    v = PyIter_Next(iter);
+    if (v == NULL)  /* stop iterator */
         return NULL;
 
     vi = PyObject_IsTrue(v);
@@ -1995,7 +2057,7 @@ btree_traverse(PyObject *iter, PyObject *tree)
     subtree = PyList_GetItem(tree, vi);
 
     if (PyList_Check(subtree) && PyList_Size(subtree) == 2)
-        return btree_traverse(iter, subtree);
+        return tree_traverse(iter, subtree);
     else
         return subtree;
 }
@@ -2013,7 +2075,7 @@ bitarray_decode(bitarrayobject *self, PyObject *tree)
 
     /* traverse binary tree and append symbols to the result list */
     res = PyList_New(0);
-    while ((symbol = btree_traverse(iter, tree)) != NULL) {
+    while ((symbol = tree_traverse(iter, tree)) != NULL) {
         if (IS_EMPTY_LIST(symbol)) {
             PyErr_SetString(PyExc_ValueError,
                             "prefix code does not match data in bitarray");
@@ -2087,6 +2149,8 @@ bitarray_methods[] = {
      setall_doc},
     {"_search",      (PyCFunction) bitarray_search,      METH_VARARGS,
      search_doc},
+    {"_search_at",   (PyCFunction) bitarray_search_at,   METH_VARARGS,
+     search_at_doc},
     {"sort",         (PyCFunction) bitarray_sort,        METH_VARARGS |
                                                          METH_KEYWORDS,
      sort_doc},
@@ -2143,7 +2207,7 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     PyObject *initial = NULL;
     idx_t nbits = 0;
     char *endianStr = "<NOT_PROVIDED>";
-    int endian = DEFAULT_ENDIAN;
+    int endian;
     static char* kwlist[] = {"initial", "endian", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
@@ -2158,13 +2222,14 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
         endian = 1;
     }
     else if (strcmp(endianStr, "<NOT_PROVIDED>") == 0) {
-        /* do nothing, keep the default value */
+        endian = DEFAULT_ENDIAN;  /* use default value */
     }
     else {
         PyErr_SetString(PyExc_ValueError,
                         "endian must be 'little' or 'big'");
         return NULL;
     }
+
     /* no arg or None */
     if (initial == NULL || initial == Py_None)
         return newbitarrayobject(type, 0, endian);
@@ -2220,7 +2285,7 @@ richcompare(PyObject *v, PyObject *w, int op)
 #define va  ((bitarrayobject *) v)
 #define wa  ((bitarrayobject *) w)
     if (va->nbits != wa->nbits && (op == Py_EQ || op == Py_NE)) {
-        /* Shortcut: if the lengths differ, the bitarrays differ */
+        /* shortcut: if the lengths differ, the bitarrays differ */
         if (op == Py_EQ)
             Py_RETURN_FALSE;
 
@@ -2228,7 +2293,7 @@ richcompare(PyObject *v, PyObject *w, int op)
     }
     /* to avoid uninitialized warning for some compilers */
     vi = wi = 0;
-    /* Search for the first index where items are different */
+    /* search for the first index where items are different */
     k = 1;
     for (i = 0; i < va->nbits && i < wa->nbits; i++) {
         vi = GETBIT(va, i);
@@ -2238,7 +2303,7 @@ richcompare(PyObject *v, PyObject *w, int op)
             break;
     }
     if (k) {
-        /* No more items to compare -- compare sizes */
+        /* no more items to compare -- compare sizes */
         idx_t vs = va->nbits;
         idx_t ws = wa->nbits;
 #undef va
@@ -2264,7 +2329,7 @@ richcompare(PyObject *v, PyObject *w, int op)
     if (op == Py_NE)
         Py_RETURN_TRUE;
 
-    /* Compare the final item using the proper operator */
+    /* compare the final item using the proper operator */
     switch (op) {
     case Py_LT: cmp = vi <  wi; break;
     case Py_LE: cmp = vi <= wi; break;
@@ -2433,16 +2498,16 @@ bits2bytes(PyObject *self, PyObject *v)
 {
     idx_t n = 0;
 
-    if (ISINDEX(v)) {
-        getIndex(v, &n);
-        if (n >= 0)
-            return PyLong_FromLongLong(BYTES(n));
-
+    if (!ISINDEX(v)) {
+        PyErr_SetString(PyExc_TypeError, "integer expected");
+        return NULL;
+    }
+    getIndex(v, &n);
+    if (n < 0) {
         PyErr_SetString(PyExc_ValueError, "positive value expected");
         return NULL;
     }
-    PyErr_SetString(PyExc_TypeError, "integer expected");
-    return NULL;
+    return PyLong_FromLongLong(BYTES(n));
 }
 
 
