@@ -152,7 +152,7 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.assertEqual(a.tolist(), [])
         self.check_obj(a)
 
-    def test_endian(self):
+    def test_endian1(self):
         a = bitarray(endian='little')
         a.fromstring('A')
         self.assertEqual(a.endian(), 'little')
@@ -165,6 +165,7 @@ class CreateObjectTests(unittest.TestCase, Util):
 
         self.assertEqual(a.tostring(), b.tostring())
 
+    def test_endian2(self):
         a = bitarray(endian='little')
         a.fromstring(' ')
         self.assertEqual(a.endian(), 'little')
@@ -179,6 +180,8 @@ class CreateObjectTests(unittest.TestCase, Util):
 
         self.assertRaises(TypeError, bitarray.__new__, bitarray, endian=0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, endian='')
+        self.assertRaises(ValueError, bitarray.__new__,
+                          bitarray, endian='foo')
 
     def test_integers(self):
         for n in range(50):
@@ -217,7 +220,7 @@ class CreateObjectTests(unittest.TestCase, Util):
             self.assertEqual(a.tolist(), lst)
             self.check_obj(a)
 
-    def test_iter(self):
+    def test_iter1(self):
         for n in range(50):
             lst = [bool(randint(0, 1)) for d in range(n)]
             a = bitarray(iter(lst))
@@ -289,7 +292,7 @@ tests.append(CreateObjectTests)
 
 class MetaDataTests(unittest.TestCase):
 
-    def test_buffer_info(self):
+    def test_buffer_info1(self):
         a = bitarray('0000111100001', endian='little')
         self.assertEqual(a.buffer_info()[1:4], (2, 'little', 3))
 
@@ -308,12 +311,14 @@ class MetaDataTests(unittest.TestCase):
         if is_py3k:
             self.assert_(isinstance(bi[4], int))
 
+    def test_buffer_info2(self):
         for n in range(50):
             bi = bitarray(n).buffer_info()
             self.assertEqual(bi[1], bits2bytes(n))
             self.assertEqual(bi[3] + n, 8 * bi[1])
             self.assert_(bi[4] >= bi[1])
 
+    def test_buffer_info3(self):
         a = bitarray(endian='little')
         self.assertEqual(a.buffer_info()[2], 'little')
 
@@ -342,7 +347,7 @@ tests.append(MetaDataTests)
 
 class SliceTests(unittest.TestCase, Util):
 
-    def test_getitem(self):
+    def test_getitem1(self):
         a = bitarray()
         self.assertRaises(IndexError, a.__getitem__,  0)
         a.append(True)
@@ -355,6 +360,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(IndexError, a.__getitem__,  2)
         self.assertRaises(IndexError, a.__getitem__, -3)
 
+    def test_getitem2(self):
         a = bitarray('1100010')
         for i, b in enumerate([True, True, False, False, False, True, False]):
             self.assertEqual(a[i], b)
@@ -362,6 +368,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(IndexError, a.__getitem__,  7)
         self.assertRaises(IndexError, a.__getitem__, -8)
 
+    def test_getitem3(self):
         a = bitarray('0100000100001')
         self.assertEQUAL(a[:], a)
         self.assert_(a[:] is not a)
@@ -372,6 +379,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(ValueError, a.__getitem__, slice(None, None, 0))
         self.assertRaises(TypeError, a.__getitem__, (1, 2))
 
+    def test_getitem4(self):
         for a in self.randombitarrays():
             aa = a.tolist()
             la = len(a)
@@ -383,7 +391,7 @@ class SliceTests(unittest.TestCase, Util):
                           self.rndsliceidx(la), step)
                 self.assertEQUAL(a[s], bitarray(aa[s], endian=a.endian()))
 
-    def test_setitem(self):
+    def test_setitem1(self):
         a = bitarray([False])
         a[0] = 1
         self.assertEqual(a.tolist(), [True])
@@ -399,11 +407,12 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(IndexError, a.__setitem__,  2, True)
         self.assertRaises(IndexError, a.__setitem__, -3, False)
 
+    def test_setitem2(self):
         for a in self.randombitarrays():
             la = len(a)
             if la == 0:
                 continue
-            i = randint(0, la-1)
+            i = randint(0, la - 1)
             aa = a.tolist()
             ida = id(a)
             val = bool(randint(0, 1))
@@ -427,13 +436,15 @@ class SliceTests(unittest.TestCase, Util):
             b[::-1] = bitarray(a)
             self.assertEqual(a.tolist()[::-1], b.tolist())
 
-        a = bitarray(5*[False])
+    def test_setitem3(self):
+        a = bitarray(5 * [False])
         a[0] = 1
         a[-2] = 1
         self.assertEqual(a, bitarray('10010'))
         self.assertRaises(IndexError, a.__setitem__,  5, 'foo')
         self.assertRaises(IndexError, a.__setitem__, -6, 'bar')
 
+    def test_setitem4(self):
         for a in self.randombitarrays():
             la = len(a)
             if la == 0: continue
@@ -468,7 +479,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertEqual(a, bitarray('11111111'))
 
 
-    def test_delitem(self):
+    def test_delitem1(self):
         a = bitarray('100110')
         del a[1]
         self.assertEqual(len(a), 5)
@@ -478,6 +489,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(IndexError, a.__delitem__,  3)
         self.assertRaises(IndexError, a.__delitem__, -4)
 
+    def test_delitem2(self):
         for a in self.randombitarrays():
             la = len(a)
             if la == 0: continue
@@ -563,7 +575,7 @@ class MiscTests(unittest.TestCase, Util):
             for i in range(len(a)):
                 self.assertEqual(a[i], b[i+1234])
 
-    def test_endianness(self):
+    def test_endianness1(self):
         a = bitarray(endian='little')
         a.frombytes(to_bytes('\x01'))
         self.assertEqual(a.to01(), '10000000')
@@ -583,6 +595,7 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(a, c)
         self.assertEqual(b, d)
 
+    def test_endianness2(self):
         a = bitarray(8, endian='little')
         a.setall(False)
         a[0] = True
@@ -593,6 +606,7 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(a.tobytes(), to_bytes('\x03 '))
         self.assertEqual(a.to01(), '1100000000000100')
 
+    def test_endianness3(self):
         a = bitarray(8, endian='big')
         a.setall(False)
         a[7] = True
@@ -603,14 +617,14 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(a.tobytes(), to_bytes('\x03 '))
         self.assertEqual(a.to01(), '0000001100100000')
 
+    def test_endianness4(self):
         a = bitarray('00100000', endian='big')
         self.assertEqual(a.tobytes(), to_bytes(' '))
-
         b = bitarray('00000100', endian='little')
         self.assertEqual(b.tobytes(), to_bytes(' '))
-
         self.assertNotEqual(a, b)
 
+    def test_endianness5(self):
         a = bitarray('11100000', endian='little')
         b = bitarray(a, endian='big')
         self.assertNotEqual(a, b)
@@ -639,7 +653,7 @@ class MiscTests(unittest.TestCase, Util):
         self.assertRaises(OverflowError, bitarray.__new__,
                           bitarray, 2**34 + 1)
 
-        a = bitarray(10**6)
+        a = bitarray(10 ** 6)
         self.assertRaises(OverflowError, a.__imul__, 17180)
 
 
@@ -798,10 +812,8 @@ class NumberTests(unittest.TestCase, Util):
                 c = n * a
                 self.assertEqual(c, bitarray(n * a.tolist(),
                                              endian=a.endian()))
-
                 self.assertEQUAL(a, b)
 
-
         a = bitarray()
         self.assertRaises(TypeError, a.__mul__, None)
 
@@ -857,7 +869,6 @@ class BitwiseTests(unittest.TestCase, Util):
 
         self.assertRaises(TypeError, a.__and__, 42)
 
-
     def test_iand(self):
         a =  bitarray('110010110')
         ida = id(a)
@@ -870,7 +881,7 @@ class BitwiseTests(unittest.TestCase, Util):
         b = bitarray('10011')
         self.assertEQUAL(a | b, bitarray('11011'))
 
-    def test_iand(self):
+    def test_ior(self):
         a =  bitarray('110010110')
         a |= bitarray('100110011')
         self.assertEQUAL(a, bitarray('110110111'))
@@ -924,7 +935,7 @@ tests.append(BitwiseTests)
 
 class SequenceTests(unittest.TestCase, Util):
 
-    def test_contains(self):
+    def test_contains1(self):
         a = bitarray()
         self.assert_(False not in a)
         self.assert_(True not in a)
@@ -937,21 +948,33 @@ class SequenceTests(unittest.TestCase, Util):
         a.append(True)
         self.assert_(0 in a)
         self.assert_(1 in a)
+
+    def test_contains2(self):
+        a = bitarray()
+        self.assertEqual(a.__contains__(1), False)
+        a.append(1)
+        self.assertEqual(a.__contains__(1), True)
+        a = bitarray('0011')
+        self.assertEqual(a.__contains__('01'), True)
+        self.assertEqual(a.__contains__('10'), False)
+
+    def test_contains3(self):
         for n in range(2, 100):
             a = bitarray(n)
             a.setall(0)
             self.assert_(False in a)
             self.assert_(True not in a)
-            a[randint(0, n-1)] = 1
+            a[randint(0, n - 1)] = 1
             self.assert_(True in a)
             self.assert_(False in a)
             a.setall(1)
             self.assert_(True in a)
             self.assert_(False not in a)
-            a[randint(0, n-1)] = 0
+            a[randint(0, n - 1)] = 0
             self.assert_(True in a)
             self.assert_(False in a)
 
+    def test_contains4(self):
         a = bitarray('011010000001')
         self.assert_('1' in a)
         self.assert_('11' in a)
@@ -1179,6 +1202,49 @@ class MethodTests(unittest.TestCase, Util):
         self.assertEqual(a.search(bitarray('0')), [])
         self.assertEqual(a.search(bitarray('1')), list(range(100)))
 
+        a = bitarray('10010101110011111001011')
+        for limit in range(10):
+            self.assertEqual(a.search('011', limit),
+                             [6, 11, 20][:limit])
+        # search is overlapping
+        self.assertEqual(a.search('111'), [7, 12, 13, 14])
+
+        self.assertRaises(ValueError, a.search, '')
+
+    def test_search_at(self):
+        a = bitarray('')
+        self.assertEqual(a._search_at(bitarray('0'), 0), None)
+        self.assertEqual(a._search_at(bitarray('1'), 0), None)
+
+        a = bitarray('1')
+        self.assertEqual(a._search_at(bitarray('0'), 0), None)
+        self.assertEqual(a._search_at(bitarray('1'), 0), 0)
+        self.assertEqual(a._search_at(bitarray('11'), 0), None)
+        self.assertRaises(ValueError, a._search_at, bitarray(), 0)
+        self.assertRaises(ValueError, a._search_at, bitarray('0'), -1)
+
+        a = bitarray('10011')
+        for s, start, res in [('0',  0, 1),    ('01',  0, 2),
+                              ('11', 1, 3),    ('000', 0, None),
+                              ('1',  0, 0),    ('1',   1, 3),
+                              ('1',  3, 3),    ('1',   4, 4),
+                              ('1',  5, None), ('11',  4, None),
+                              ('10011', 0, 0), ('100110', 0, None)]:
+            self.assertEqual(a._search_at(bitarray(s), start), res)
+
+    def test_itersearch(self):
+        if is_py3k:
+            return
+        a = bitarray('10011')
+        it = a.itersearch('1')
+        self.assertEqual(it.next(), 0)
+        self.assertEqual(it.next(), 3)
+        self.assertEqual(it.next(), 4)
+        self.assertRaises(StopIteration, it.next)
+        it = a.itersearch('')
+        self.assertRaises(ValueError, it.next)
+
+    def test_search2(self):
         a = bitarray('10011')
         for s, res in [('0',     [1, 2]),  ('1', [0, 3, 4]),
                        ('01',    [2]),     ('11', [3]),
@@ -1186,15 +1252,21 @@ class MethodTests(unittest.TestCase, Util):
                        ('011',   [2]),     ('0011', [1]),
                        ('10011', [0]),     ('100111', [])]:
             self.assertEqual(a.search(s), res)
+            self.assertEqual([p for p in a.itersearch(s)], res)
             b = bitarray(s)
             self.assertEqual(a.search(b), res)
             self.assertEqual(a.search(list(b)), res)
             self.assertEqual(a.search(tuple(b)), res)
 
+    def test_search3(self):
         a = bitarray('10010101110011111001011')
-        for limit in range(10):
-            self.assertEqual(a.search('011', limit),
-                             [6, 11, 20][:limit])
+        for s, res in [('011', [6, 11, 20]),
+                       ('111', [7, 12, 13, 14]),
+                       ('1011', [5, 19]),
+                       ('100', [0, 9, 16])]:
+            self.assertEqual(a.search(s), res)
+            self.assertEqual(list(a.itersearch(s)), res)
+            self.assertEqual([p for p in a.itersearch(s)], res)
 
 
     def test_fill(self):
@@ -1237,10 +1309,12 @@ class MethodTests(unittest.TestCase, Util):
         a = bitarray('1101000')
         a.sort(reverse=True)
         self.assertEqual(a, bitarray('1110000'))
-
-        a = bitarray('1101000')
+        a.sort(reverse=False)
+        self.assertEqual(a, bitarray('0000111'))
         a.sort(True)
         self.assertEqual(a, bitarray('1110000'))
+        a.sort(False)
+        self.assertEqual(a, bitarray('0000111'))
 
         self.assertRaises(TypeError, a.sort, 'A')
 
@@ -1344,27 +1418,23 @@ class MethodTests(unittest.TestCase, Util):
             val = randint(0, 1)
             b = a
             b.setall(val)
-            self.assertEqual(b, bitarray(len(b)*[val]))
+            self.assertEqual(b, bitarray(len(b) * [val]))
             self.assert_(a is b)
             self.check_obj(b)
 
 
     def test_bytereverse(self):
-        a = bitarray()
-        a.bytereverse()
-        self.assertEqual(a, bitarray())
-
-        a = bitarray('1011')
-        a.bytereverse()
-        self.assertEqual(a, bitarray('0000'))
-
-        a = bitarray('111011')
-        a.bytereverse()
-        self.assertEqual(a, bitarray('001101'))
-
-        a = bitarray('11101101')
-        a.bytereverse()
-        self.assertEqual(a, bitarray('10110111'))
+        for x, y in [('', ''),
+                     ('1', '0'),
+                     ('1011', '0000'),
+                     ('111011', '001101'),
+                     ('11101101', '10110111'),
+                     ('000000011', '100000000'),
+                     ('11011111' '00100000' '000111',
+                      '11111011' '00000100' '001110')]:
+            a = bitarray(x)
+            a.bytereverse()
+            self.assertEqual(a, bitarray(y))
 
         for i in range(256):
             a = bitarray()
@@ -1455,8 +1525,8 @@ class StringTests(unittest.TestCase, Util):
         self.assertRaises(TypeError, a.unpack, foo=to_bytes('b'))
 
         for a in self.randombitarrays():
-            self.assertEqual(a.unpack(to_bytes('0'), to_bytes('1')).decode(),
-                             a.to01())
+            self.assertEqual(a.unpack(to_bytes('0'), to_bytes('1')),
+                             to_bytes(a.to01()))
 
             b = bitarray()
             b.pack(a.unpack())
@@ -1851,7 +1921,6 @@ tests.append(PrefixCodeTests)
 def pages():
     if sys.platform != 'linux2':
         return 0
-
     dat = open('/proc/%i/statm' % os.getpid()).read()
     return int(dat.split()[0])
 
@@ -1868,7 +1937,7 @@ def check_memory_leaks(verbosity):
     i = 0
     runner = unittest.TextTestRunner(verbosity=verbosity)
     while True:
-        print('Run', i)
+        print('Run: %d' % i)
         r = runner.run(suite)
         if i % 1 == 0:
             fo = open(logfile, 'a')
@@ -1896,6 +1965,6 @@ if __name__ == '__main__':
 
 else:
     from bitarray import __version__
-    print('bitarray is installed in:', os.path.dirname(__file__))
-    print('bitarray version:', __version__)
+    print('bitarray is installed in: ' + os.path.dirname(__file__))
+    print('bitarray version: ' + __version__)
     print(sys.version)
